/**
 * vue-inertia-composable
 *
 * @description Inertia for Vue2 Composition API.
 * @author Logue <logue@hotmail.co.jp>
 * @copyright 2022 By Masashi Yoshikawa All rights reserved.
 * @license MIT
 * @version 0.3.4
 * @see {@link https://github.com/logue/vue-inertia-composable}
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi'), require('ziggy-js'), require('@inertiajs/inertia')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi', 'ziggy-js', '@inertiajs/inertia'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.InertiaComposable = {}, global.VueDemi, global.ziggy, global.Inertia));
})(this, (function (exports, vueDemi, ziggy, inertia) { 'use strict';

  /**
   * Alternate of Inertia Link Component
   *
   * @experimental
   */
  const InertiaLink = vueDemi.defineComponent({
    name: 'InertiaLink',
    props: {
      as: {
        type: String,
        default: 'a',
      },
      data: {
        type: Object,
        default: () => {},
      },
      href: {
        type: String,
        default: '',
      },
      method: {
        type: String,
        default: 'get',
      },
      replace: {
        type: Boolean,
        default: false,
      },
      preserveScroll: {
        type: Boolean,
        default: false,
      },
      preserveState: {
        type: Boolean,
        default: false,
      },
      only: {
        type: Array,
        default: () => [],
      },
      headers: {
        type: Object,
        default: () => {},
      },
      queryStringArrayFormat: {
        type: String,
        default: 'brackets',
      },
    },
    render() {
      const as = this.$props.as.toLowerCase() || 'a';
      const method = this.$props.method.toLocaleLowerCase() || 'get';
      const attrs = this.$attrs;
      const [href, data] = inertia.mergeDataIntoQueryString(
        method,
        this.$props.href || '',
        this.$props.data,
        this.$props.queryStringArrayFormat
      );
      if (as === 'a' && method !== 'get') {
        console.warn(
          `Creating POST/PUT/PATCH/DELETE <a> links is discouraged as it causes "Open Link in New Tab/Window" accessibility issues.\n\nPlease specify a more appropriate element using the "as" attribute. For example:\n\n<inertia-link href="${href}" method="${method}" as="button">...</inertia-link>`
        );
      }
      return vueDemi.h(
        as,
        {
          ...this.$attrs,
          ...(as === 'a' ? { href } : {}),
          onClick: event => {
            if (!inertia.shouldIntercept(event)) {
              return;
            }
            event.preventDefault();
            inertia.Inertia.visit(href, {
              data: data,
              method: method,
              replace: this.$props.replace,
              preserveScroll: this.$props.preserveScroll,
              preserveState: this.$props.preserveState ?? method !== 'get',
              only: this.$props.only,
              headers: this.$props.headers,
              onCancelToken: attrs.onCancelToken || (() => undefined),
              onBefore: attrs.onBefore || (() => undefined),
              onStart: attrs.onStart || (() => undefined),
              onProgress: attrs.onProgress || (() => undefined),
              onFinish: attrs.onFinish || (() => undefined),
              onCancel: attrs.onCancel || (() => undefined),
              onSuccess: attrs.onSuccess || (() => undefined),
              onError: attrs.onError || (() => undefined),
            });
          },
        },
        this.$slots.default
      );
    },
  });
  const installInertiaLink = app => app.component('InertiaLink', InertiaLink);
  if (typeof window !== 'undefined' && window.Vue) {
    // @ts-ignore
    window.Vue.use(InertiaLink);
  }

  function useHeadManager() {
    const instance = vueDemi.getCurrentInstance();
    if (instance) {
      return instance.proxy.$headManager;
    } else {
      warn();
    }
    return void 0;
  }
  function usePage() {
    const instance = vueDemi.getCurrentInstance();
    if (instance) {
      return instance.proxy.$page;
    } else {
      warn();
    }
    return void 0;
  }
  function useInertia() {
    const instance = vueDemi.getCurrentInstance();
    if (instance) {
      return instance.proxy.$inertia;
    } else {
      warn();
    }
    return void 0;
  }
  function useForm(args) {
    const instance = vueDemi.getCurrentInstance();
    if (instance) {
      return instance.proxy.$inertia.form(args);
    } else {
      warn();
    }
    return void 0;
  }
  function route(name, params, absolute, config) {
    const instance = vueDemi.getCurrentInstance();
    if (instance) {
      return instance.proxy.route(name, params, absolute, config);
    }
    return ziggy(name, params, absolute, config);
  }
  const warn = () => console.warn(
    `[Inertia Composable] getCurrentInstance() returned null. Method must be called at the top of a setup() function.`
  );
  if (typeof window !== "undefined") {
    window.route = route;
  }

  exports.InertiaLink = InertiaLink;
  exports.install = installInertiaLink;
  exports.route = route;
  exports.useForm = useForm;
  exports.useHeadManager = useHeadManager;
  exports.useInertia = useInertia;
  exports.usePage = usePage;

  Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

}));
